@@ -1,4 +1,4 @@
-__version__ = '0.4.3'
+__version__ = '0.4.4'
 
 # amalgamate exclude jupyter_kernel parser_table parser_test_table pyghooks
 # amalgamate exclude winutils wizard
@@ -15,30 +15,28 @@ else:
         _sys.modules['xonsh.lazyasd'] = __amalgam__
         lazyjson = __amalgam__
         _sys.modules['xonsh.lazyjson'] = __amalgam__
-        pretty = __amalgam__
-        _sys.modules['xonsh.pretty'] = __amalgam__
-        timings = __amalgam__
-        _sys.modules['xonsh.timings'] = __amalgam__
         ansi_colors = __amalgam__
         _sys.modules['xonsh.ansi_colors'] = __amalgam__
         codecache = __amalgam__
         _sys.modules['xonsh.codecache'] = __amalgam__
-        openpy = __amalgam__
-        _sys.modules['xonsh.openpy'] = __amalgam__
+        lazyimps = __amalgam__
+        _sys.modules['xonsh.lazyimps'] = __amalgam__
         platform = __amalgam__
         _sys.modules['xonsh.platform'] = __amalgam__
-        teepty = __amalgam__
-        _sys.modules['xonsh.teepty'] = __amalgam__
+        pretty = __amalgam__
+        _sys.modules['xonsh.pretty'] = __amalgam__
+        timings = __amalgam__
+        _sys.modules['xonsh.timings'] = __amalgam__
         jobs = __amalgam__
         _sys.modules['xonsh.jobs'] = __amalgam__
         parser = __amalgam__
         _sys.modules['xonsh.parser'] = __amalgam__
+        teepty = __amalgam__
+        _sys.modules['xonsh.teepty'] = __amalgam__
         tokenize = __amalgam__
         _sys.modules['xonsh.tokenize'] = __amalgam__
         tools = __amalgam__
         _sys.modules['xonsh.tools'] = __amalgam__
-        vox = __amalgam__
-        _sys.modules['xonsh.vox'] = __amalgam__
         ast = __amalgam__
         _sys.modules['xonsh.ast'] = __amalgam__
         contexts = __amalgam__
@@ -49,10 +47,10 @@ else:
         _sys.modules['xonsh.dirstack'] = __amalgam__
         foreign_shells = __amalgam__
         _sys.modules['xonsh.foreign_shells'] = __amalgam__
-        inspectors = __amalgam__
-        _sys.modules['xonsh.inspectors'] = __amalgam__
         lexer = __amalgam__
         _sys.modules['xonsh.lexer'] = __amalgam__
+        openpy = __amalgam__
+        _sys.modules['xonsh.openpy'] = __amalgam__
         proc = __amalgam__
         _sys.modules['xonsh.proc'] = __amalgam__
         xontribs = __amalgam__
@@ -63,6 +61,8 @@ else:
         _sys.modules['xonsh.environ'] = __amalgam__
         history = __amalgam__
         _sys.modules['xonsh.history'] = __amalgam__
+        inspectors = __amalgam__
+        _sys.modules['xonsh.inspectors'] = __amalgam__
         base_shell = __amalgam__
         _sys.modules['xonsh.base_shell'] = __amalgam__
         replay = __amalgam__
