@@ -1,31 +1,31 @@
 # -*- coding: utf-8 -*-
 """Aliases for the xonsh shell."""
-
-from argparse import ArgumentParser, Action
-import builtins
-from collections.abc import MutableMapping, Iterable, Sequence
 import os
 import sys
 import shlex
+import argparse
+import builtins
+import collections.abc as abc
 
+from xonsh.lazyasd import lazyobject
 from xonsh.dirstack import cd, pushd, popd, dirs, _get_cwd
 from xonsh.environ import locate_binary
 from xonsh.foreign_shells import foreign_shell_data
 from xonsh.jobs import jobs, fg, bg, clean_jobs
 from xonsh.history import history_main
-from xonsh.platform import ON_ANACONDA, ON_DARWIN, ON_WINDOWS, scandir
+from xonsh.platform import (ON_ANACONDA, ON_DARWIN, ON_WINDOWS, ON_FREEBSD,
+    scandir)
 from xonsh.proc import foreground
 from xonsh.replay import replay_main
 from xonsh.timings import timeit_alias
 from xonsh.tools import (XonshError, argvquote, escape_windows_cmd_string,
                          to_bool)
-from xonsh.vox import Vox
 from xonsh.xontribs import xontribs_main
 from xonsh.xoreutils import _which
 from xonsh.completers._aliases import completer_alias
 
 
-class Aliases(MutableMapping):
+class Aliases(abc.MutableMapping):
     """Represents a location to hold and look up aliases."""
 
     def __init__(self, *args, **kwargs):
@@ -43,7 +43,7 @@ class Aliases(MutableMapping):
         val = self._raw.get(key)
         if val is None:
             return default
-        elif isinstance(val, Iterable) or callable(val):
+        elif isinstance(val, abc.Iterable) or callable(val):
             return self.eval_alias(val, seen_tokens={key})
         else:
             msg = 'alias of {!r} has an inappropriate type: {!r}'
@@ -91,7 +91,8 @@ class Aliases(MutableMapping):
         builtin function and if alias is only a single command.
         """
         word = line.split(' ', 1)[0]
-        if word in builtins.aliases and isinstance(self.get(word), Sequence):
+        if word in builtins.aliases and isinstance(self.get(word),
+                                                   abc.Sequence):
             word_idx = line.find(word)
             expansion = ' '.join(self.get(word))
             line = line[:word_idx] + expansion + line[word_idx+len(word):]
@@ -152,15 +153,10 @@ def xonsh_exit(args, stdin=None):
     return None, None
 
 
-_SOURCE_FOREIGN_PARSER = None
-
-
-def _ensure_source_foreign_parser():
-    global _SOURCE_FOREIGN_PARSER
-    if _SOURCE_FOREIGN_PARSER is not None:
-        return _SOURCE_FOREIGN_PARSER
+@lazyobject
+def _SOURCE_FOREIGN_PARSER():
     desc = "Sources a file written in a foreign shell language."
-    parser = ArgumentParser('source-foreign', description=desc)
+    parser = argparse.ArgumentParser('source-foreign', description=desc)
     parser.add_argument('shell', help='Name or path to the foreign shell')
     parser.add_argument('files_or_code', nargs='+',
                         help='file paths to source or code in the target '
@@ -203,14 +199,12 @@ def _ensure_source_foreign_parser():
     parser.add_argument('--seterrpostcmd', default=None, dest='seterrpostcmd',
                         help='command(s) to set exit-on-error after all'
                              'other commands.')
-    _SOURCE_FOREIGN_PARSER = parser
     return parser
 
 
 def source_foreign(args, stdin=None):
     """Sources a file written in a foreign shell language."""
-    parser = _ensure_source_foreign_parser()
-    ns = parser.parse_args(args)
+    ns = _SOURCE_FOREIGN_PARSER.parse_args(args)
     if ns.prevcmd is not None:
         pass  # don't change prevcmd if given explicitly
     elif os.path.isfile(ns.files_or_code[0]):
@@ -301,19 +295,19 @@ def xexec(args, stdin=None):
         return (None, 'xonsh: exec: no args specified\n', 1)
 
 
-_BANG_N_PARSER = None
+@lazyobject
+def _BANG_N_PARSER():
+    parser = argparse.ArgumentParser('!n', usage='!n <n>',
+                description="Re-runs the nth command as specified in the "
+                            "argument.")
+    parser.add_argument('n', type=int, help='the command to rerun, may be '
+                                            'negative')
+    return parser
 
 
 def bang_n(args, stdin=None):
     """Re-runs the nth command as specified in the argument."""
-    global _BANG_N_PARSER
-    if _BANG_N_PARSER is None:
-        parser = _BANG_N_PARSER = ArgumentParser('!n', usage='!n <n>',
-                    description="Re-runs the nth command as specified in the argument.")
-        parser.add_argument('n', type=int, help='the command to rerun, may be negative')
-    else:
-        parser = _BANG_N_PARSER
-    ns = parser.parse_args(args)
+    ns = _BANG_N_PARSER.parse_args(args)
     hist = builtins.__xonsh_history__
     nhist = len(hist)
     n = nhist + ns.n if ns.n < 0 else ns.n
@@ -330,7 +324,7 @@ def bang_bang(args, stdin=None):
     return bang_n(['-1'])
 
 
-class AWitchAWitch(Action):
+class AWitchAWitch(argparse.Action):
     SUPPRESS = '==SUPPRESS=='
     def __init__(self, option_strings, version=None, dest=SUPPRESS,
                  default=SUPPRESS, **kwargs):
@@ -351,7 +345,7 @@ def which(args, stdin=None, stdout=None, stderr=None):
     `which` match.
     """
     desc = "Parses arguments to which wrapper"
-    parser = ArgumentParser('which', description=desc)
+    parser = argparse.ArgumentParser('which', description=desc)
     parser.add_argument('args', type=str, nargs='+',
                         help='The executables or aliases to search for')
     parser.add_argument('-a','--all', action='store_true', dest='all',
@@ -463,12 +457,6 @@ def trace(args, stdin=None):
         pass
 
 
-def vox(args, stdin=None):
-    """Runs Vox environment manager."""
-    vox = Vox()
-    return vox(args, stdin=stdin)
-
-
 def showcmd(args, stdin=None):
     """usage: showcmd [-h|--help|cmd args]
 
@@ -518,7 +506,6 @@ def make_default_aliases():
         'scp-resume': ['rsync', '--partial', '-h', '--progress', '--rsh=ssh'],
         'showcmd': showcmd,
         'ipynb': ['jupyter', 'notebook', '--no-browser'],
-        'vox': vox,
         'which': which,
         'xontrib': xontribs_main,
         'completer': completer_alias
@@ -573,6 +560,11 @@ def make_default_aliases():
             default_aliases['sudo'] = sudo
     elif ON_DARWIN:
         default_aliases['ls'] = ['ls', '-G']
+    elif ON_FREEBSD:
+        default_aliases['grep'] = ['grep', '--color=auto']
+        default_aliases['egrep'] = ['egrep', '--color=auto']
+        default_aliases['fgrep'] = ['fgrep', '--color=auto']
+        default_aliases['ls'] = ['ls', '-G']
     else:
         default_aliases['grep'] = ['grep', '--color=auto']
         default_aliases['egrep'] = ['egrep', '--color=auto']
