@@ -32,16 +32,17 @@ import sys
 import threading
 import traceback
 import warnings
-from contextlib import contextmanager
-from subprocess import CalledProcessError
+import contextlib
+import subprocess
 
 # adding further imports from xonsh modules is discouraged to avoid circular
 # dependencies
-from xonsh.lazyasd import LazyObject, LazyDict
+from xonsh.lazyasd import LazyObject, LazyDict, lazyobject
 from xonsh.platform import (has_prompt_toolkit, scandir, DEFAULT_ENCODING,
                             ON_LINUX, ON_WINDOWS, PYTHON_VERSION_INFO)
 
 
+
 @functools.lru_cache(1)
 def is_superuser():
     if ON_WINDOWS:
@@ -75,7 +76,7 @@ class XonshBlockError(XonshError):
         self.locs = locs
 
 
-class XonshCalledProcessError(XonshError, CalledProcessError):
+class XonshCalledProcessError(XonshError, subprocess.CalledProcessError):
     """Raised when there's an error with a called process
 
     Inherits from XonshError and subprocess.CalledProcessError, catching
@@ -88,7 +89,7 @@ class XonshCalledProcessError(XonshError, CalledProcessError):
         try:
             for line in !(ls):
                 print(line)
-        except CalledProcessError as error:
+        except subprocess.CalledProcessError as error:
             print("Error in process: {}.format(error.completed_command.pid))
 
     This also handles differences between Python3.4 and 3.5 where
@@ -739,7 +740,7 @@ def on_main_thread():
     return threading.current_thread() is threading.main_thread()
 
 
-@contextmanager
+@contextlib.contextmanager
 def swap(namespace, name, value, default=NotImplemented):
     """Swaps a current variable name in a namespace for another value, and then
     replaces it when the context is exited.
@@ -762,20 +763,6 @@ def is_int(x):
     return isinstance(x, int)
 
 
-def is_int_as_str(x):
-    """
-    Tests if something is an integer
-    If not a string to begin with is automatically false
-    """
-    if isinstance(x, str):
-        try:
-            int(x)
-        except ValueError:
-            return False
-        return True
-    return False
-
-
 def is_float(x):
     """Tests if something is a float"""
     return isinstance(x, float)
@@ -791,25 +778,6 @@ def is_slice(x):
     return isinstance(x, slice)
 
 
-def is_slice_as_str(x):
-    """
-    Tests if a str is a slice
-    If not a string to begin with is automatically false
-    """
-    if isinstance(x, str):
-        if ':' in x:
-            x = x.strip('[]()')
-            try:
-                slice(*(int(x) if len(x) > 0 else None
-                        for x in x.split(':')))
-            except ValueError:
-                return False
-        else:
-            return False
-        return True
-    return False
-
-
 def is_callable(x):
     """Tests if something is callable"""
     return callable(x)
@@ -923,7 +891,9 @@ def bool_to_str(x):
     return '1' if x else ''
 
 
-_BREAKS = frozenset(['b', 'break', 's', 'skip', 'q', 'quit'])
+_BREAKS = LazyObject(lambda: frozenset(['b', 'break', 's', 'skip',
+                                        'q', 'quit']),
+                     globals(), '_BREAKS')
 
 
 def to_bool_or_break(x):
@@ -953,21 +923,55 @@ def bool_or_int_to_str(x):
     return bool_to_str(x) if is_bool(x) else str(x)
 
 
-def ensure_int_or_slice(x):
-    """Makes sure that x is list-indexable."""
+
+@lazyobject
+def SLICE_REG():
+    return re.compile(r'(?P<start>(?:-\d)?\d*):(?P<end>(?:-\d)?\d*):?(?P<step>(?:-\d)?\d*)')
+
+def ensure_slice(x):
+    """Convert a string or int to a slice."""
     if x is None:
         return slice(None)
-    elif is_int(x) or is_slice(x):
-        return x
-    # must have a string from here on
-    if is_slice_as_str(x):
+    try:
+        x = int(x)
+        s = slice(x, x+1)
+    except ValueError:
         x = x.strip('[]()')
-        return slice(*(int(x) if len(x) > 0 else None for x in x.split(':')))
-    elif is_int_as_str(x):
-        return int(x)
+        m = SLICE_REG.fullmatch(x)
+        if m:
+            groups = (int(i) if i else None for i in m.groups())
+            s = slice(*groups)
+        else:
+            raise ValueError('cannot convert {!r} to slice'.format(x))
+    except TypeError:
+        raise TypeError('ensure_slice() argument must be a string or a number not {}'.format(type(x)))
+    return s
+
+
+def is_slice_as_str(x):
+    """
+    Test if string x is a slice. If not a string return False.
+    """
+    try:
+        x = x.strip('[]()')
+        m = SLICE_REG.fullmatch(x)
+        if m:
+            return True
+    except AttributeError:
+        pass
     return False
 
 
+def is_int_as_str(x):
+    """
+    Test if string x is an integer. If not a string return False.
+    """
+    try:
+        return x.isdecimal()
+    except AttributeError:
+        return False
+
+
 def is_string_set(x):
     """Tests if something is a set of strings"""
     return (isinstance(x, abc.Set) and
@@ -1270,7 +1274,7 @@ def _get_color_indexes(style_map):
                 rgb = (int(attr.color[0:2], 16),
                        int(attr.color[2:4], 16),
                        int(attr.color[4:6], 16))
-            except:
+            except Exception:
                 rgb = None
             yield token, index, rgb
 
@@ -1328,7 +1332,8 @@ def intensify_colors_on_win_setter(enable):
     environment variable.
     """
     enable = to_bool(enable)
-    delattr(builtins.__xonsh_shell__.shell.styler, 'style_name')
+    if hasattr(builtins, '__xonsh_shell__'):
+        delattr(builtins.__xonsh_shell__.shell.styler, 'style_name')
     return enable
 
 
@@ -1431,29 +1436,22 @@ def check_for_partial_string(x):
         return (string_indices[-2], string_indices[-1], starting_quote[-1])
 
 
-# expandvars is a modified version of os.path.expandvars from the Python 3.5.1
-# source code (root/Lib/ntpath.py, line 353)
-
-def _is_in_env(name):
-    env = builtins.__xonsh_env__
-    return name in env._d or name in env._defaults
-
-
-def _get_env_string(name):
-    env = builtins.__xonsh_env__
-    value = env.get(name)
-    ensurer = env.get_ensurer(name)
-    if ensurer.detype is bool_to_str:
-        value = ensure_string(value)
-    else:
-        value = ensurer.detype(value)
-    return value
+# regular expressions for matching enviroment variables
+# i.e $FOO, ${'FOO'}
+@lazyobject
+def POSIX_ENVVAR_REGEX():
+    pat = r"""\$({(?P<quote>['"])|)(?P<envvar>\w+)((?P=quote)}|(?:\1\b))"""
+    return re.compile(pat)
 
+if ON_WINDOWS:
+    # i.e %FOO%
+    @lazyobject
+    def WINDOWS_ENVVAR_REGEX():
+        return re.compile(r"%(?P<envvar>\w+)%")
 
 def expandvars(path):
     """Expand shell variables of the forms $var, ${var} and %var%.
-    Unknown variables are left unchanged.
-    """
+    Unknown variables are left unchanged."""
     env = builtins.__xonsh_env__
     if isinstance(path, bytes):
         path = path.decode(encoding=env.get('XONSH_ENCODING'),
@@ -1461,91 +1459,21 @@ def expandvars(path):
     elif isinstance(path, pathlib.Path):
         # get the path's string representation
         path = str(path)
-    if '$' not in path and (not ON_WINDOWS or '%' not in path):
-        return path
-    varchars = string.ascii_letters + string.digits + '_-'
-    quote = '\''
-    percent = '%'
-    brace = '{'
-    rbrace = '}'
-    dollar = '$'
-    res = path[:0]
-    index = 0
-    pathlen = len(path)
-    while index < pathlen:
-        c = path[index:index+1]
-        if c == quote:   # no expansion within single quotes
-            path = path[index + 1:]
-            pathlen = len(path)
-            try:
-                index = path.index(c)
-                res += c + path[:index + 1]
-            except ValueError:
-                res += c + path
-                index = pathlen - 1
-        elif c == percent and ON_WINDOWS:  # variable or '%'
-            if path[index + 1:index + 2] == percent:
-                res += c
-                index += 1
-            else:
-                path = path[index+1:]
-                pathlen = len(path)
-                try:
-                    index = path.index(percent)
-                except ValueError:
-                    res += percent + path
-                    index = pathlen - 1
-                else:
-                    var = path[:index]
-                    if _is_in_env(var):
-                        value = _get_env_string(var)
-                    else:
-                        value = percent + var + percent
-                    res += value
-        elif c == dollar:  # variable or '$$'
-            if path[index + 1:index + 2] == dollar:
-                res += c
-                index += 1
-            elif path[index + 1:index + 2] == brace:
-                path = path[index+2:]
-                pathlen = len(path)
-                try:
-                    index = path.index(rbrace)
-                except ValueError:
-                    res += dollar + brace + path
-                    index = pathlen - 1
-                else:
-                    var = path[:index]
-                    try:
-                        var = eval(var, builtins.__xonsh_ctx__)
-                        if _is_in_env(var):
-                            value = _get_env_string(var)
-                        elif var is Ellipsis:
-                            value = dollar + brace + '...' + rbrace
-                        else:
-                            value = dollar + brace + var + rbrace
-                    except:
-                        value = dollar + brace + var + rbrace
-                    res += value
-            else:
-                var = path[:0]
-                index += 1
-                c = path[index:index + 1]
-                while c and c in varchars:
-                    var += c
-                    index += 1
-                    c = path[index:index + 1]
-                if _is_in_env(var):
-                    value = _get_env_string(var)
-                else:
-                    value = dollar + var
-                res += value
-                if c:
-                    index -= 1
-        else:
-            res += c
-        index += 1
-    return res
+    if ON_WINDOWS and '%' in path:
+        for match in WINDOWS_ENVVAR_REGEX.finditer(path):
+            name = match.group('envvar')
+            if name in env:
+                ensurer = env.get_ensurer(name)
+                value = ensurer.detype(env[name])
+                path = WINDOWS_ENVVAR_REGEX.sub(value, path, count=1)
+    if '$' in path:
+        for match in POSIX_ENVVAR_REGEX.finditer(path):
+            name = match.group('envvar')
+            if name in env:
+                ensurer = env.get_ensurer(name)
+                value = ensurer.detype(env[name])
+                path = POSIX_ENVVAR_REGEX.sub(value, path, count=1)
+    return path
 
 #
 # File handling tools
